import {
  Button,
  Dialog,
  DialogActions,
  DialogContent,
  DialogContentText,
  DialogTitle,
} from "@mui/material";
import React, {
  PropsWithChildren,
  createContext,
  useContext,
  useRef,
} from "react";

interface ConfirmHook {
  confirm: (
    message: string,
    title?: string,
    confirmButton?: string
  ) => Promise<boolean>;
}

const ConfirmContext = createContext<ConfirmHook | null>(null);

export function ConfirmDialogProvider(
  p: PropsWithChildren
): React.ReactElement {
  const [open, setOpen] = React.useState(false);

  const [title, setTitle] = React.useState<string | undefined>(undefined);
  const [message, setMessage] = React.useState("");
  const [confirmButton, setConfirmButton] = React.useState<string | undefined>(
    undefined
  );

  const cb = React.useRef<null | ((a: boolean) => void)>(null);

  const handleClose = (confirm: boolean) => {
    setOpen(false);

    if (cb.current !== null) cb.current(confirm);
    cb.current = null;
  };

  const hook: ConfirmHook = {
    confirm: (message, title, confirmButton) => {
      setTitle(title);
      setMessage(message);
      setConfirmButton(confirmButton);
      setOpen(true);

      return new Promise((res) => {
        cb.current = res;
      });
    },
  };

  return (
    <>
      <ConfirmContext.Provider value={hook}>
        {p.children}
      </ConfirmContext.Provider>

      <Dialog
        open={open}
        onClose={() => handleClose(false)}
        aria-labelledby="alert-dialog-title"
        aria-describedby="alert-dialog-description"
      >
        {title && <DialogTitle id="alert-dialog-title">{title}</DialogTitle>}
        <DialogContent>
          <DialogContentText id="alert-dialog-description">
            {message}
          </DialogContentText>
        </DialogContent>
        <DialogActions>
          <Button onClick={() => handleClose(false)} autoFocus>
            Annuler
          </Button>
          <Button onClick={() => handleClose(true)} color="error">
            {confirmButton ?? "Confirmer"}
          </Button>
        </DialogActions>
      </Dialog>
    </>
  );
}

export function useConfirm(): ConfirmHook {
  return useContext(ConfirmContext)!;
}
